@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -116,7 +116,7 @@ void global_scope_t::parse_init(path init_file)
   parsing_context.get_current().journal = session().journal.get();
   parsing_context.get_current().scope   = &report();
 
-  if (session().journal->read(parsing_context) > 0 ||
+  if (session().journal->read(parsing_context, NO_HASHES) > 0 ||
       session().journal->auto_xacts.size() > 0 ||
       session().journal->period_xacts.size() > 0) {
     throw_(parse_error, _f("Transactions found in initialization file '%1%'")
@@ -431,13 +431,11 @@ global_scope_t::read_command_arguments(scope_t& scope, strings_list args)
 
 void global_scope_t::normalize_session_options()
 {
-#if LOGGING_ON
   INFO("Initialization file is " << HANDLER(init_file_).str());
   INFO("Price database is " << session().HANDLER(price_db_).str());
 
   foreach (const path& pathname, session().HANDLER(file_).data_files)
     INFO("Journal file is " << pathname.string());
-#endif // LOGGING_ON
 }
 
 expr_t::func_t global_scope_t::look_for_precommand(scope_t&      scope,
@@ -501,9 +499,7 @@ void handle_debug_options(int argc, char * argv[])
       }
       else if (std::strcmp(argv[i], "--verbose") == 0 ||
                std::strcmp(argv[i], "-v") == 0) {
-#if LOGGING_ON
         _log_level = LOG_INFO;
-#endif
       }
       else if (i + 1 < argc && std::strcmp(argv[i], "--init-file") == 0) {
         _init_file = argv[i + 1];
